package zero1hd.rhythmbullet.audio;

import java.io.File;
import java.io.IOException;

import javax.sound.sampled.UnsupportedAudioFileException;

import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.files.FileHandle;

import zero1hd.rhythmbullet.audio.processor.MP3PCMProcessor;
import zero1hd.rhythmbullet.audio.processor.PCMProcessor;
import zero1hd.rhythmbullet.audio.processor.WAVPCMProcessor;

public class VisualizableMusic implements Music {
	private int windowSize = 1024;
	private OnCompletionListener ocl;
	private boolean looping;
	private RhythmBulletAudioThread rat;
	private File musicFile;
	private PCMProcessor sampleProcessor;
	
	public VisualizableMusic(FileHandle file) {
		musicFile = file.file();
		if (musicFile.getName().toLowerCase().endsWith("wav")) {
			try {
				sampleProcessor = new WAVPCMProcessor(musicFile, windowSize);
			} catch (IOException | UnsupportedAudioFileException e) {
				e.printStackTrace();
			}
		} else if (musicFile.getName().toLowerCase().endsWith("mp3")) {
			sampleProcessor = new MP3PCMProcessor(musicFile, windowSize);
		}
	}
	
	@Override
	public void play() {
		if (rat == null) {
			rat = new RhythmBulletAudioThread(windowSize, this);
			rat.setOnCompletionListener(ocl);
			rat.start();
		} else {
			rat.play();
		}
	}

	@Override
	public void pause() {
		rat.pause();
	}

	@Override
	public void stop() {
		rat.terminate();
		rat = null;
	}

	@Override
	public boolean isPlaying() {
		return rat.isPlaying();
	}

	@Override
	public void setLooping(boolean isLooping) {
		looping = isLooping;
		
	}

	@Override
	public boolean isLooping() {
		return looping;
	}

	@Override
	public void setVolume(float volume) {
		sampleProcessor.setVolume(volume);
	}

	@Override
	public float getVolume() {
		return sampleProcessor.getVolume();
	}

	@Override
	public void setPan(float pan, float volume) {
		sampleProcessor.setPan(pan);
		sampleProcessor.setVolume(volume);
	}

	@Override
	public void setPosition(float position) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public float getPosition() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public void dispose() {
		// TODO Auto-generated method stub
	}

	@Override
	public void setOnCompletionListener(OnCompletionListener listener) {
		this.ocl = listener;
		if (rat != null) {
			rat.setOnCompletionListener(listener);
		}
	}
	
	protected PCMProcessor getSampleProcessor() {
		return sampleProcessor;
	}
}
