package zero1hd.rhythmbullet.audio.processor;

import com.badlogic.gdx.utils.Disposable;

public interface PCMProcessor extends Disposable {
	
	/**
	 * Called once, contains the initiation to the stream, only called when play-back begins.
	 * Not thread safe as it should be the first thing to be called during read process.
	 */
	public void initiate();
	
	/**
	 * @return number of channels
	 */
	public int getChannels();
	
	/**
	 * @return sample rate
	 */
	public int getSampleRate();
	
	/**
	 * <b>Thread safe</b>
	 * Reads samples (NOT FRAMES) with interwoven data for stereo.
	 * stored in 16 bit format (first 8 are the first byte of data while the second 8 are the second byte of data that composes a short value) 
	 * @param pcm the array the samples should fill
	 * @param syncObj the object that this object should use to synchronize multiple threads.
	 * @return the amount of samples read.
	 */
	public int readSamples(short[] pcm, Object syncObj);
	
	/**
	 * <b>Thread safe</b>
	 * ranges from 0 to 1.
	 * @param volume the volume to set
	 */
	public void setVolume(float volume);
	
	/**
	 * <b>Thread safe</b>
	 * @return the volume ranging from 0 to 1
	 */
	public float getVolume();
	
	/**
	 * <b>Thread safe</b>
	 * sets the pan from the left channel to the right.
	 * ranges from -1 to 1.
	 * Default is 0, -1 is left, 1 is right.
	 * @param pan
	 */
	public void setPan(float pan);
	
	/**
	 * <b>Thread safe</b>
	 * Returns the pan value from -1 to 1.
	 * see {@link #setPan(float)} for more information.
	 * @return the pan value.
	 */
	public float getPan();
}
