package zero1hd.rhythmbullet.audio.processor;

import java.io.File;
import java.io.IOException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

public class WAVPCMProcessor implements PCMProcessor {
	private int channels;
	private int sampleRate;
	private byte[] buffer;
	private File file;
	private AudioInputStream audioInputStream;
	private volatile float volume, pan;
	private volatile boolean initiated;
	
	public WAVPCMProcessor(File musicFile, int windowSize) throws IOException, UnsupportedAudioFileException {
		this.file = musicFile;
		AudioFormat format = AudioSystem.getAudioFileFormat(file).getFormat();
		channels = format.getChannels();
		sampleRate = (int) format.getSampleRate();
		volume = 1f;
	}
	
	@Override
	public void initiate() {
		try {
			audioInputStream = AudioSystem.getAudioInputStream(file);
		} catch (UnsupportedAudioFileException | IOException e) {
			e.printStackTrace();
		}
		buffer = new byte[audioInputStream.getFormat().getFrameSize()];
		initiated = true;
	}
	
	public int getChannels() {
		return channels;
	}
	
	public int getSampleRate() {
		return sampleRate;
	}
	
	@Override
	public int readSamples(short[] pcm, Object syncObj) {
		if (initiated) {
			synchronized (syncObj) {
				int framesRead = 0;
				for (int sampleID = 0; sampleID < pcm.length; sampleID++) {
					try {
						if (audioInputStream.read(buffer) > 0) {
							pcm[sampleID] = (short) ((buffer[1] << 8) + (buffer[0] & 0x00ff));
							pcm[sampleID] *= volume * (Math.min(1-pan, 1));
							if (audioInputStream.getFormat().getChannels() > 1) {
								short secondChan = (short) ((buffer[3] << 8) + (buffer[2] & 0x00ff));
								secondChan *= volume * (Math.min(1+pan, 1));
								sampleID++;
								pcm[sampleID] = secondChan;
							}
							framesRead++;
						}
					} catch (IOException e) {
						e.printStackTrace();
					}
					
				}
				return framesRead;
			}
		} else {
			throw new IllegalStateException("Stream has not been initialized.");
		}
	}

	@Override
	public void setVolume(float volume) {
		this.volume = volume;
	}

	@Override
	public float getVolume() {
		return volume;
	}

	@Override
	public void setPan(float pan) {
		this.pan = pan;
	}

	@Override
	public float getPan() {
		return pan;
	}

	@Override
	public void dispose() {
		// TODO Auto-generated method stub
		
	}
}